-- TODO
--
-- * Adjust the average-over time, perhaps 60 seconds is more fitting? Needs testing
--   in a raid environment. We don't want the values jumping around like crazy, better
--   to show something more stable, even if it's a "lie".
--

-- How many seconds of data to use when calculating the average bytes per second.
local AVERAGE_OVER = 10
local spammers = {}

local L = LibStub("AceLocale-3.0"):GetLocale("Broker_CMA")

local text = "%.1f"
local broker = LibStub("LibDataBroker-1.1")
-- Icon shamelessly borrowed from the Tango Project
local icon = "Interface\\AddOns\\Broker_CMA\\icon"
local CMA = broker:NewDataObject(L["Transfer stats"], {
	icon = icon,
	suffix = L["bps"],
	type = "data source",
})
if not CMA then return end
local f = CreateFrame("Frame")
local totalElapsed = 0
local lastSecond = 0
local highestAverage = 0
local highestPeak = 0

local r = setmetatable({}, {__add = function(t, n)
	t[#t + 1] = n
	if #t > AVERAGE_OVER then table.remove(t, 1) end
	local x = 0
	for i, v in next, t do x = x + v end
	return x
end})
f:SetScript("OnUpdate", function(self, elapsed)
	totalElapsed = totalElapsed + elapsed
	if totalElapsed >= 1 then
		local lastPeriod = r + lastSecond
		local average = lastPeriod / #r
		CMA.value = text:format(average)
		CMA.text = format("%s %s", CMA.value, CMA.suffix)

		-- Calculate highscores!
		if lastSecond > highestPeak then highestPeak = lastSecond end
		-- But only calculate highest average if we've gathered enough data
		if #r == AVERAGE_OVER and average > highestAverage then highestAverage = average end

		lastSecond = 0
		totalElapsed = 0
	end
end)
f:SetScript("OnEvent", function(self, event, prefix, message, distribution, sender)
	-- 40 is a guesstimate message overhead stolen from ChatThrottleLib by Mikk.
	-- This "total message size" estimate is probably (hopefully) a bit bigger than it should be.
	local size = prefix:len() + message:len() + distribution:len() + sender:len() + 40
	lastSecond = lastSecond + size
	spammers[sender] = (spammers[sender] or 0) + size
end)
f:RegisterEvent("CHAT_MSG_ADDON")
f:Show()

local tmp = {}
local function spamSort(a, b) return spammers[a] > spammers[b] end
function CMA.OnTooltipShow(tt)
	if not tt or not tt.AddLine or not tt.AddDoubleLine then return end
	tt:AddLine(L["Transfer stats"])
	tt:AddDoubleLine(L["Current average"], ("%s %s"):format(CMA.value or "?", CMA.suffix or "?"), 1, 1, 1, 0, 1, 0)
	tt:AddDoubleLine(L["Average over"], L["%d sec"]:format(AVERAGE_OVER), 1, 1, 1, 0, 1, 0)
	tt:AddDoubleLine(L["Highest average"], ("%.1f %s"):format(highestAverage, L["bps"]), 1, 1, 1, 0, 1, 0)
	tt:AddDoubleLine(L["Peak second"], L["%d bytes"]:format(highestPeak), 1, 1, 1, 0, 1, 0)
	tt:AddLine(" ")
	for k in pairs(spammers) do tmp[#tmp + 1] = k end
	if #tmp > 0 then
		table.sort(tmp, spamSort)
		tt:AddLine(L["Most data sent"])
		for i = 1, 3 do
			if not tmp[i] then break end
			tt:AddDoubleLine(L["%d. %s"]:format(i, tmp[i]), L["%d bytes"]:format(spammers[tmp[i]]), 1, 1, 1, 0, 1, 0)
		end
		tt:AddLine(" ")
		wipe(tmp)
	end
	tt:AddLine(L["Transfer statistics for the addon communication channel."], 0.2, 1, 0.2, 1)
end

-- For debugging
--_G.BrokerCMATable = r

